@@ -40,7 +40,6 @@ from ._version import kernel_protocol_version
 
 CONTROL_PRIORITY = 1
 SHELL_PRIORITY = 10
-ABORT_PRIORITY = 20
 
 
 class Kernel(SingletonConfigurable):
@@ -182,10 +181,6 @@ class Kernel(SingletonConfigurable):
         # Set the parent message for side effects.
         self.set_parent(idents, msg)
         self._publish_status('busy')
-        if self._aborting:
-            self._send_abort_reply(self.control_stream, msg, idents)
-            self._publish_status('idle')
-            return
 
         header = msg['header']
         msg_type = header['msg_type']
@@ -233,7 +228,10 @@ class Kernel(SingletonConfigurable):
         self.set_parent(idents, msg)
         self._publish_status('busy')
 
-        if self._aborting:
+        msg_type = msg['header']['msg_type']
+
+        # Only abort execute requests
+        if self._aborting and msg_type == 'execute_request':
             self._send_abort_reply(stream, msg, idents)
             self._publish_status('idle')
             # flush to ensure reply is sent before
@@ -241,8 +239,6 @@ class Kernel(SingletonConfigurable):
             stream.flush(zmq.POLLOUT)
             return
 
-        msg_type = msg['header']['msg_type']
-
         # Print some info about this message and leave a '--->' marker, so it's
         # easier to trace visually the message chain when debugging.  Each
         # handler prints its message at the end.
@@ -792,16 +788,11 @@ class Kernel(SingletonConfigurable):
             stream.flush()
         self._aborting = True
 
-        self.schedule_dispatch(
-            ABORT_PRIORITY,
-            self._dispatch_abort,
-        )
+        def stop_aborting(f):
+            self.log.info("Finishing abort")
+            self._aborting = False
 
-    @gen.coroutine
-    def _dispatch_abort(self):
-        self.log.info("Finishing abort")
-        yield gen.sleep(self.stop_on_error_timeout)
-        self._aborting = False
+        self.io_loop.add_future(gen.sleep(self.stop_on_error_timeout), stop_aborting)
 
     def _send_abort_reply(self, stream, msg, idents):
         """Send a reply to an aborted request"""
@@ -817,7 +808,7 @@ class Kernel(SingletonConfigurable):
         )
 
     def _no_raw_input(self):
-        """Raise StdinNotImplentedError if active frontend doesn't support
+        """Raise StdinNotImplementedError if active frontend doesn't support
         stdin."""
         raise StdinNotImplementedError("raw_input was called, but this "
                                        "frontend does not support stdin.")
@@ -827,7 +818,7 @@ class Kernel(SingletonConfigurable):
 
         Raises
         ------
-        StdinNotImplentedError if active frontend doesn't support stdin.
+        StdinNotImplementedError if active frontend doesn't support stdin.
         """
         if not self._allow_stdin:
             raise StdinNotImplementedError(
@@ -848,7 +839,7 @@ class Kernel(SingletonConfigurable):
 
         Raises
         ------
-        StdinNotImplentedError if active frontend doesn't support stdin.
+        StdinNotImplementedError if active frontend doesn't support stdin.
         """
         if not self._allow_stdin:
             raise StdinNotImplementedError(
